"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.existVariableInTestFunction = void 0;
var util_1 = require("./util");
exports.existVariableInTestFunction = {
    meta: {
        fixable: 'code',
        messages: {
            'UndefinedVariables': 'The test function argument contains a variable that is not defined.'
        }
    },
    create: function (context) {
        var sourceCode = context.getSourceCode();
        return {
            TaggedTemplateExpression: function (node) {
                var results = (0, util_1.getTestEachVariables)(node, sourceCode);
                if (results === undefined) {
                    return;
                }
                var header = results.header;
                // 親の引数に第1引数にLiteralがなければ終了
                var parent = node.parent;
                if (parent.type !== 'CallExpression' || parent.arguments.length !== 2) {
                    return;
                }
                if (parent.arguments[1].type !== 'ArrowFunctionExpression' ||
                    parent.arguments[1].params.length !== 1 ||
                    parent.arguments[1].params[0].type !== 'ObjectPattern') {
                    return;
                }
                // テスト関数の引数一覧を取得
                var properties = parent.arguments[1].params[0].properties.map(function (m) {
                    if (m.type === 'Property' && m.key.type === 'Identifier') {
                        return m.key.name;
                    }
                    return undefined;
                }).filter(function (f) { return f !== undefined; });
                // 使用している引数が、テンプレート文字列に存在するかチェック
                var isNotExist = properties.map(function (m) { return header.includes(m); }).includes(false);
                if (isNotExist) {
                    context.report({ node: parent.arguments[1], messageId: 'UndefinedVariables' });
                }
            }
        };
    }
};
